
/*********************************************************************
 * check_hugefiles.c
 * (c) 2004 Oliver Schroeder
 * www.o-schroeder.de
 **********************************************************************/

#if defined (HAVE_CONFIG_H)
#	include "config.h"
#endif

/*
 * FIXME:	additional checks in the source
 */

#if defined (HAVE_STDIO_H)
#	include <stdio.h>
#endif
#if defined (HAVE_DIRENT_H)
#	include <dirent.h>
#endif
#if defined (HAVE_CTYPE_H)
#	include <ctype.h>
#endif
#if defined (HAVE_STRING_H)
#	include <string.h>
#endif
#include <stdlib.h>
#if defined (HAVE_SYS_TYPES_H)
#	include <sys/types.h>
#endif
#if defined (HAVE_SYS_STAT_H)
#	include <sys/stat.h>
#endif
#if !defined (STATE_CRITICAL)	/* from nagios common.h */
#	define OK		1
#	define ERROR		-1
#	define CRIT_ERROR	-2
#	define TRUE		1
#	define FALSE		0
#	define STATE_OK		0
#	define STATE_WARNING	1
#	define STATE_CRITICAL	2
#	define MAX_INPUT_BUFFER	2048
#endif
/*************************************************
 * define size type
 * FIXME: autoconf
 **************************************************/
typedef long long MY_SIZE;

/*************************************************
 * globals
 **************************************************/
MY_SIZE	g_size;
char	g_path[MAX_INPUT_BUFFER];
int	g_verbose = FALSE;
char	g_error_msg[MAX_INPUT_BUFFER];

/*************************************************
 * prototypes
 **************************************************/
int process_arguments ( int argc, char* argv[] );
int check_files ( char* path );
void print_help ( void );
void print_usage ( void );

/*********************************************************************
 * Main
 **********************************************************************/
int
main ( int argc, char* argv[] )
{
	int result;
	
	process_arguments (argc, argv);
	if (g_verbose)
	{
		printf ("searching '%s' for files larger than %lld bytes...\n",
		  g_path, g_size);
	}
	result = check_files (g_path);
	if (result == CRIT_ERROR)
	{
		printf ("CRITICAL - %s", g_error_msg);
		return (STATE_CRITICAL);
	}
	if (result == ERROR)
	{
		printf ("WARNING - %s\n", g_error_msg);
		return (STATE_WARNING);
	}
	if (result)
	{
		printf ("WARNING - %d files found ", result);
		printf ("with size >%lld byte ", g_size);
		printf ("beneath '%s'\n", g_path);
		return (STATE_WARNING);
	}
	printf ("OK - 0 files found\n");
	return (STATE_OK);
}

/*********************************************************************
 * the actual workhorse
 **********************************************************************/
int
check_files ( char* path )
{
	int		error;
	extern int	errno;
	char		complete_path [MAX_INPUT_BUFFER];
	DIR*		dirp;
	struct stat	stats;
	struct dirent*	dir_entry;
	int		num_files;

	num_files = 0;
	dirp = opendir (path);
	if (dirp == 0)
	{
		sprintf (g_error_msg, "could not read directory '%s'!\n", path);
		return (CRIT_ERROR);
	}
	errno = 0;
	error = stat (path, & stats);
	if (error != 0)
	{
		sprintf (g_error_msg, "%s -> %s\n", path, strerror (errno));
		return (CRIT_ERROR);
	}
	while ((dir_entry = readdir (dirp)) != 0)
	{
		if ((! strcmp (dir_entry->d_name, "."))
		||  (! strcmp (dir_entry->d_name, "..")))
		{
			continue;
		}
		strcpy (complete_path, path);
		if (path[strlen (path) - 1] != '/')
		{
			strcat (complete_path, "/");
		}
		strcat (complete_path, dir_entry->d_name);
		errno = 0;
		error = stat (complete_path, & stats);
		if (error != 0)
		{
			if (g_verbose)
			{
				printf ("%s -> %s\n", complete_path,
				  strerror (errno));
			}
			continue;
		}
		if (S_ISDIR (stats.st_mode))
		{
			error = check_files (complete_path);
			if (error > 0)
				num_files += error;
			if (error < 0)
				return (error);
		}
		if (stats.st_size > g_size)
		{
			num_files++;
			if (g_verbose)
			{
				printf ("found '%s' (%lld Byte)\n",
				  complete_path, (long long)stats.st_size);
			}
		}
	} /* while */
	return (num_files);
}

/*********************************************************************
 * parse the command line
 **********************************************************************/
int
process_arguments ( int argc, char* argv[] )
{
	int i, n, i2;

	if ((argc > 1) && (strcmp (argv[1], "-h") == 0))
	{
		print_help ();
		exit (0);
	}
	if ((argc < 3) || (argc > 4))
	{
		print_help ();
		exit (ERROR);
	}
	n = 1;
	for (i = 1; i < argc; i++)
	{
		if (strcmp (argv[i], "-v") == 0)
		{
			g_verbose = TRUE;
		}
		else if (strcmp (argv[i], "-h") == 0)
		{
			print_help ();
		}
		else if (n == 1)
		{
			strncpy (g_path, argv[i], MAX_INPUT_BUFFER);
			n++;
		}
		else if (n == 2)
		{
			g_size = atoll (argv[i]);
			n++;
			i2 = 0;
			while ((argv[i][i2] != '\0') && (isdigit (argv[i][i2])))
				i2++;
			if (argv[i][i2] == '\0') 
				continue;
			switch (argv[i][i2])
			{
				case 'G':
				case 'g':
					g_size *= 1024*1024*1024;
					break;
				case 'M':
				case 'm':
					g_size *= 1024*1024;
					break;
				case 'K':
				case 'k':
					g_size *= 1024;
					break;
				default:
					printf ("unknown '%c'!\n",
					  argv[i][i2]);
					break;
			}
		}
		else
		{
			print_usage ();
			printf ("unknown option '%s'\n", argv[i]);
			exit (ERROR);
		}
	}
	return (0);
}

/*********************************************************************
 * print help screen
 **********************************************************************/
void
print_help ( void )
{
	printf (
"\n%s v%s (c) 2004 Oliver Schroeder\n\n"
"This plugin will search <path> for files with a filesize greater than\n"
"<size> and generate a NAGIOS compliant alert.\n", PACKAGE, VERSION);
	print_usage ();
	printf ("Parameters:\n"
" <path>     the path to search\n"
" <size>     the filesize threshold\n"
"            can have modifiers 'g','G','m','M','k','K'\n"
"            eg: 100k resulting in 102400\n"
" -v         if given, report all files found\n\n");
}

/*********************************************************************
 * print usage
 **********************************************************************/
void
print_usage ( void )
{
	printf ("\nUsage: %s [-v|-h] <path> <size>\n\n", PACKAGE);
}
